<?php

declare(strict_types=1);

namespace mirzaev\beejee\models;

use mirzaev\beejee\models\model;

use PDO;

/**
 * Модель заданий
 *
 * @package mirzaev\beejee\models
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 */
final class tasksModel extends model
{
    /**
     * Создать запись
     *
     * @param string $name Имя
     * @param string $email Почта
     * @param string $task Задание
     *
     * @return array
     */
    public function create(string $name, string $email, string $task): array
    {
        // Инициализация
        $request = $this->db->prepare('INSERT INTO `tasks` (`name`, `email`, `task`) VALUES (:name, :email, :task)');

        // Параметры
        $request->bindValue(':name', $name, PDO::PARAM_STR);
        $request->bindValue(':email', $email, PDO::PARAM_STR);
        $request->bindValue(':task', $task, PDO::PARAM_STR);

        // Отправка
        $request->execute();

        return (array) $request->fetchAll(PDO::FETCH_ASSOC);
    }

    /**
     * Прочитать запись
     *
     * @param int $page Страница
     * @param int $limit Количество заданий
     * @param string $sort Сортировка
     *
     * @return array
     */
    public function read(int $page, int $limit, string $sort): array
    {
        // Нормализация сортировки по белому списку
        if ($sort === 'id' || $sort === 'name' || $sort === 'email' || $sort === 'task') {
            // Инициализация
            $request = $this->db->prepare('SELECT * FROM `tasks` ORDER BY `' . $sort . '` LIMIT :page, :limit');

            // Параметры
            $request->bindValue(':page', $page, PDO::PARAM_INT);
            $request->bindValue(':limit', $limit, PDO::PARAM_INT);

            // Отправка
            $request->execute();

            return (array) $request->fetchAll(PDO::FETCH_ASSOC);
        }
        return [];
    }

    /**
     * Обновить запись
     *
     * @param int $name Идентификатор
     * @param string $name Имя
     * @param string $email Почта
     * @param string $task Задание
     * @param string $task Статус о завершении
     *
     * @return array
     */
    public function update(int $id, string $name = null, string $email = null, string $task = null, string $completed = null): array
    {
        // Инициализация строки запроса
        $request  = 'UPDATE `tasks` SET';

        // Проверка на то, что уже стоит какое-то значение
        // Нужно для того, чтобы ставить запятые
        $comma = false;

        if (!is_null($name)) {
            // Имя
            $request .= '`name` = :name';
            $params[':name'] = $name;
            $comma = true;
        }

        if (!is_null($email)) {
            // Почта
            $request .=  ($comma ? ', ' : '') . '`email` = :email';
            $params[':email'] = $email;
            $comma = true;
        }

        if (!is_null($task)) {
            // Задание
            $request .= ($comma ? ', ' : '') . '`task` = :task';
            $params[':task'] = $task;
            $comma = true;
        }

        if (!is_null($completed) && $completed == 1) {
            // Статус завершения
            $request .=  ($comma ? ', ' : '') . '`completed` = NOT `completed`';
        }

        $request .= ' WHERE `id` = :id';
        $params[':id'] = $id;

        echo $request;
        // Инициализация запроса
        $request = $this->db->prepare($request);

        // Отправка
        $request->execute($params);

        return (array) $request->fetchAll(PDO::FETCH_ASSOC);
    }

    /**
     * Удалить запись
     *
     * @param int $name Идентификатор
     *
     * @return bool
     */
    public function delete(int $id): bool
    {
        // Инициализация запроса
        $request = 'DELETE FROM `tasks` WHERE `id` = :id';

        // Параметры
        $params = [
            ':id' => $id
        ];

        // Запрос
        return (bool) $this->db->prepare($request)->execute($params);
    }

    /**
     * Подсчитать количество заданий
     *
     * @return int
     */
    public function count(): int
    {
        // Запрос к базе данных
        $request = ceil($this->db->query('SELECT count(*) as count FROM `tasks`')->fetch()['count'] / 3);

        return (int) (empty($request) ? 1 : $request);
    }
}
